# Copyright (c) 2005-2006, Samuel Schluep, ETH Zurich and Infrae
# All rights reserved. See also LICENSE.txt

# Zope
from zope.interface import implements
# Silva
from Products.Silva.adapters import adapter
# Silva Glossary
from Products.SilvaGlossary.adapters.interfaces import IGlossarizable
from Products.SilvaGlossary.helpers import glossaryLinkHelper


class GlossarizableAdapter(adapter.Adapter):
    """Default adapter class to search for glossary entries

    Does nothing, the method returns an empty dictionary.
    """
    implements(IGlossarizable)

    def getGlossaryEntries(self):
        """Returns an empty dictionary"""
        return {}


class DocumentGlossarizableAdapter(GlossarizableAdapter):
    """Adapter to search for glossary entries in Silva Documents"""

    def getGlossaryEntries(self):
        """Get the glossary entries for Silva Document

        Can be used for Silva Document like content types. Gets the glossary
        entries dictionary of the viewable version.
        The key of the dictionary is the absolute path of the associated
        glossary entry. The value is the Glossary Entry object itself.
        """
        version = self.context.get_viewable()
        if version is None:
            return {}
        return IGlossarizable(version).getGlossaryEntries()


class DocumentVersionGlossarizableAdapter(GlossarizableAdapter):
    """Adapter to search for glossary entries in Silva Document Versions"""

    def getGlossaryEntries(self):
        """Get the glossary entries as a dictionary for Silva Document Version

        Can be used for Silva Document Version like content types.
        The key of the dictionary is the absolute path of the associated
        glossary entry. The value is the Glossary Entry object itself.
        """
        version = self.context
        glossaryEntries = {}
        docElement = version.content.firstChild
        nodes = docElement.getElementsByTagName('link')
        for node in nodes:
            path = node.getAttribute('url')
            url, obj = glossaryLinkHelper(version, path)
            if obj != None and \
               obj.meta_type == 'Silva Glossary Entry':
                uid = '/'.join(obj.getPhysicalPath())
                glossaryEntries[uid] = obj
        return glossaryEntries


class GhostGlossarizableAdapter(GlossarizableAdapter):
    """Adapter to search for ghosted objects (IGhost)"""

    def getGlossaryEntries(self):
        """Get the glossary entries as a dictionary for Ghosts

        Can be used for ghost objects that implement IGhost. The method
        will look for the target object and call the adapter specific
        to that object.
        The key of the dictionary is the absolute path of the associated
        glossary entry. The value is the Glossary Entry object itself.
        """
        ghost_version = self.context.get_viewable()
        if ghost_version is None:
            return {}
        target = ghost_version.get_haunted()
        if target is None:
            return {}
        glossarizable = IGlossarizable(target)
        return glossarizable.getGlossaryEntries()


class ContainerGlossarizableAdapter(GlossarizableAdapter):
    """Adapter to get the indexes for containers"""
    def getGlossaryEntries(self):
        index = self.context.index
        return  IGlossarizable(index).getGlossaryEntries()

