# Copyright (c) 2005-2006, Samuel Schluep, ETH Zurich and Infrae
# All rights reserved. See also LICENSE.txt

"""Install module for Silva Glossary
"""

# Zope
from Acquisition import aq_base
# Silva
from Products.Silva.install import add_fss_directory_view
from Products import SilvaDocument
# SilvaGlossary
from Products.SilvaGlossary import GlossaryEntry


_widgets_dir = 'widgets'
_zmi_widgets_id = 'service_widgets_silva_glossary'
_zmi_widgets_id_silva_document = 'service_widgets'


def install(root):
    # create the core views from filesystem
    add_fss_directory_view(root.service_views,
                           'SilvaGlossary', __file__, 'views')

    # include the silva glossary globals directory
    add_fss_directory_view(root, 'globals_silva_glossary', __file__,
                           'globals_silva_glossary')

    # also register views
    registerViews(root.service_view_registry)

    # configure XML widgets
    configureXMLWidgets(root)

    # security
    root.manage_permission('Add Silva Glossary Entrys',
                           ['Author', 'Editor', 'ChiefEditor', 'Manager'])
    root.manage_permission('Add Silva Glossarys',
                           ['Author', 'Editor', 'ChiefEditor', 'Manager'])

    # metadata mapping
    mapping = root.service_metadata.getTypeMapping()
    mapping.editMappings('', [
        {'type': 'Silva Glossary Entry',
         'chain': 'silva-content, silva-extra'},
        {'type': 'Silva Glossary',
         'chain': 'silva-content, silva-extra'},
        ])

    # install the glossary renderer  service
    if not hasattr(root, 'service_silva_glossary'):
        root.manage_addProduct['SilvaGlossary'] \
            .manage_addGlossaryService(
                'service_silva_glossary',
                'Silva Glossary Configuration')


def uninstall(root):
    unregisterViews(root.service_view_registry)
    unconfigureXMLWidgets(root)
    root.service_views.manage_delObjects(['SilvaGlossary'])
    if hasattr(root, 'globals_silva_glossary'):
        root.manage_delObjects(['globals_silva_glossary'])
    # uninstall metadata mapping?
    if hasattr(root, 'service_silva_glossary'):
        # XXX this can happen always if service_resources is
        # assumed to be just there in the future (like service_views)
        root.manage_delObjects(['service_silva_glossary'])


def is_installed(root):
    return hasattr(root.service_views, 'SilvaGlossary')


def registerViews(reg):
    """Register core views on registry.
    """
    # edit
    reg.register('edit','Silva Glossary Entry',
        ['edit', 'Asset', 'GlossaryEntry'])
    reg.register('edit','Silva Glossary',
        ['edit', 'Content', 'Glossary'])
    # public
    reg.register('public','Silva Glossary Entry', 
        ['public', 'GlossaryEntry', 'view'])
    reg.register('public','Silva Glossary',
        ['public', 'Glossary', 'view'])
    # add
    reg.register('add','Silva Glossary Entry',
        ['add', 'GlossaryEntry'])
    reg.register('add','Silva Glossary',
        ['add', 'Glossary'])
    # preview
    reg.register('public','Silva Glossary Entry', 
        ['public', 'GlossaryEntry', 'preview'])
    reg.register('public','Silva Glossary',
        ['public', 'Glossary', 'preview'])
    

def unregisterViews(reg):
    for meta_type in ['Silva Glossary Entry', 'Silva Glossary']:
        reg.unregister('edit', meta_type)
        reg.unregister('public', meta_type)
        reg.unregister('add', meta_type)
        reg.unregister('preview', meta_type)
        
def configureXMLWidgets(root):
    """Configure XMLWidgets

    Redirects the behaviour of standard Silva Document widgets to include
    glossary functionallity.
    """
    # create the widget editor views from filesystem (redirect widgets)
    add_fss_directory_view(root, _zmi_widgets_id, __file__, _widgets_dir)
    # add the services for XMLWidgets
    redirectWidgets(root)


def unconfigureXMLWidgets(root):
    """Reset XMLWidgets configurations
    
    Resets rendering widget redirections.
    """
    # reset (redirect widgets)
    resetWidgets(root)
    if getattr( aq_base(root), _zmi_widgets_id, None):
        # '_zmi_widgets_id' might be missing in versions prior to 0.3 (upgrade)
        root.manage_delObjects([_zmi_widgets_id])


def redirectWidgets(root):
    """Redirects the rendering widgets of the Silva Document root element.
    
    The Silva Glossary doc widgets will include the glossary entries and
    the JavaScript code neede for the tooltip behavior.
    """
    redirectDocViewer(root)
    redirectDocPreviewer(root)


def redirectDocViewer(root):
    """Redirects the viewer widget of the Silva Document 'doc' element.
    
    The Silva Glossary doc widget will include the glossary entries and
    the JavaScript code neede for the tooltip behavior.
    """
    if getattr(aq_base(root), 'service_doc_viewer', None):
        # XXX FIXME: the redirect will not be installed in case of an 
        # upgrade, needs a second manual refresh
        wr = root.service_doc_viewer
        wr._node_map['doc'] = (_zmi_widgets_id, 'top', 'doc', 'mode_view')
        wr._node_map = wr._node_map


def redirectDocPreviewer(root):
    """Redirects the previewer widget of the Silva Document 'doc' element.
    
    The Silva Glossary doc widget will include the glossary entries and
    the JavaScript code neede for the tooltip behavior.
    """
    if getattr(aq_base(root), 'service_doc_previewer', None):
        # XXX FIXME: the redirect will not be installed in case of an 
        # upgrade, needs a second manual refresh
        wr = root.service_doc_previewer
        wr._node_map['doc'] = (_zmi_widgets_id, 'top', 'doc', 'mode_view')
        wr._node_map = wr._node_map


def resetWidgets(root):
    """Resets the widget redirection

    Resets the widget redirection to the Silva Document standard behavior.
    """
    if getattr( aq_base(root), 'service_doc_viewer', None):
        wr = root.service_doc_viewer
        wr._node_map['doc'] = (
            _zmi_widgets_id_silva_document, 'top', 'doc', 'mode_view')
        wr._node_map = wr._node_map
    if getattr( aq_base(root), 'service_doc_previewer', None):
        wr = root.service_doc_previewer
        wr._node_map['doc'] = (
        _zmi_widgets_id_silva_document, 'top', 'doc', 'mode_view')
        wr._node_map = wr._node_map


